package es.aranova.mhp.base;

import java.awt.Container;
import java.awt.event.KeyListener;
import java.util.HashMap;
import java.util.Map;

public abstract class PageContainer extends Container implements KeyListener {
	private static Map _pages = new HashMap();
	private static Map _templates = new HashMap();
	private PageContainer _parentPage;
	private PageContainer _actualPage;

	private PageContainer getHomePage() {
		if (_parentPage != null)
			return _parentPage.getHomePage();
		return this;
	}

	public void goBack() {
		if (_parentPage._parentPage == null) {
			((MainApp)_parentPage).destroyXlet(true);
			return;
		}
		goTo(_parentPage.getClass().getName(), false);
		_parentPage._actualPage = null;
		_actualPage = null;
	}

	public void goTo(final String name) {
		goTo(name, true);
	}
	public void goTo(final String name, boolean reset) {
		System.out.println("GOTO... " + name);
		final PageContainer page;
		final TemplateContainer template;
		if (!_pages.containsKey(name)) {
			page = createPage(name);
			System.out.println("Nueva pagina... " + name);
			if (!_templates.containsKey(page.getTemplate())) {
				template = createTemplate(page.getTemplate());
				System.out.println("Nuevo template... " + page.getTemplate());
			} else {
				template = (TemplateContainer)_templates.get(page.getTemplate());
				System.out.println("Template en uso... " + page.getTemplate());
			}
			template.add(page);
			page.init();
			page.setVisible(false);
		} else {
			page = (PageContainer)_pages.get(name);
			if (reset) {
				System.out.println("RESET");
				page.reset();
			}
			template = (TemplateContainer)_templates.get(page.getTemplate());
		}
		if (_parentPage != null) {
			this.setVisible(false);
			System.out.println("Oculto pagina anterior");
		}
		template.setAppName(page.getAppName());
		if (this.getTemplate() != null) {
			if (!this.getTemplate().equals(page.getTemplate())) {
				((TemplateContainer)_templates.get(getTemplate())).setVisible(false);
				template.setVisible(true);
				System.out.println("Nuevo tamplate visible");
			} else {
				System.out.println("Mismo template");
			}
		} else {
			template.setVisible(true);
			System.out.println("Template visible");
		}
		page.setVisible(true);
		System.out.println("Pagina visible");
		_actualPage = page;
		getHomePage().invalidate();
		getHomePage().repaint();
		repaint();
	}

	protected String getAppName() {
		return _parentPage.getAppName();
	}
	public String getTemplate() {
		return _parentPage.getTemplate();
	}

	private TemplateContainer createTemplate(final String name) {
		final TemplateContainer template;
		try {
			Class clazz = Class.forName(name);
			template = (TemplateContainer) clazz.newInstance();
			getHomePage().add(template);
			_templates.put(name, template);
			return template;
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	private PageContainer createPage(final String name) {
		final PageContainer page;
		try {
			Class clazz = Class.forName(name);
			page = (PageContainer) clazz.newInstance();
			page._parentPage = this;
			_pages.put(name, page);
			return page;
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	protected abstract void init();
	protected abstract void reset();

	/**
	 * @return Returns the actualPage.
	 */
	public PageContainer getActualPage() {
		return _actualPage;
	}
}
